{ ----------------------------------------------------------------------------
  MsgDlg.pas               Copyright  1998-99 by DithoSoft Software Solutions
  ----------------------------------------------------------------------------
  This unit contains the new MsgDlg function, which works just like Delphis
  MessageDlg function. However, in addition to "design improvements" the
  message sounds are also played depending on the type of message box.
  ----------------------------------------------------------------------------
  Version 1.00
          Initial release.
  ----------------------------------------------------------------------------
  homepage: http://dithosoft.cjb.net
  mail:     thorsten.dittmar@debitel.net
  ---------------------------------------------------------------------------- }
unit NewMsgDlg;

interface

uses
  Dialogs;

function MsgDlg(const Msg: string; AType: TMsgDlgType; AButtons: TMsgDlgButtons;
                HelpCtx: Longint): Word;

implementation

uses Windows, Classes, Forms, Controls, Graphics, ExtCtrls, StdCtrls;

var
  { Array of sounds }
  MsgDlgSounds : Array[TMsgDlgType] of Integer =
                 (MB_ICONEXCLAMATION, MB_ICONHAND, MB_ICONASTERISK,
                  MB_ICONQUESTION, 0);

  { Array of captions }
  MsgDlgCaptions: Array[TMsgDlgType] of String =
                  ('Warning','Error','Information','Confirm','');

  { Array of button captions }
  MsgDlgButtonCaptions : Array[TMsgDlgBtn] of String =
                         ('Yes','No','OK','Cancel','Abort','Retry','Ignore',
                          'All','No to all','Yes to all','Help');

  { Names of the buttons }
  MsgDlgButtonNames: array[TMsgDlgBtn] of string =
                     ('Yes','No','OK','Cancel','Abort','Retry','Ignore','All',
                      'NoToAll','YesToAll','Help');

  { Array of icons }
  MsgDlgIcons: array[TMsgDlgType] of PChar =
               (IDI_EXCLAMATION,IDI_HAND,IDI_ASTERISK,IDI_QUESTION,nil);

  { Array of results }
  MsgDlgResults: array[TMsgDlgBtn] of Integer = (
    mrYes, mrNo, mrOk, mrCancel, mrAbort, mrRetry, mrIgnore, mrAll, mrNoToAll,
    mrYesToAll, 0);

{ Dynamically created form -> Message Dialog }
type
  TMessageDialog = class(TForm)
  private
    FHelpContext : LongInt;
    procedure   HelpButtonClick(Sender: TObject);
    function    GetDialogUnitsX: Integer;
    function    GetDialogUnitsY: Integer;
  public
    constructor CreateNew(AOwner: TComponent);
    property    HelpContext: LongInt read FHelpContext write FHelpContext;
    property    DialogUnitsX: Integer read GetDialogUnitsX;
    property    DialogUnitsY: Integer read GetDialogUnitsY;
  end;

constructor TMessageDialog.CreateNew(AOwner: TComponent);
var
  Metrics: TNonClientMetrics;
begin
  inherited CreateNew(AOwner);

  { Set the Font to the standard font for message boxes }
  Metrics.cbSize := SizeOf(Metrics);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS,0,@Metrics,0) then
    Font.Handle := CreateFontIndirect(Metrics.lfMessageFont);
end;

procedure TMessageDialog.HelpButtonClick(Sender: TObject);
begin
  { Pass the help context to the application to run the help system }
  Application.HelpContext(FHelpContext);
end;

function TMessageDialog.GetDialogUnitsX: Integer;
var
  i     : Integer;
  Buffer: array[0..51] of Char;
  Size  : TSize;
begin
  { Create an array of all small and capital letters }
  for i := 0 to 25 do begin
    Buffer[i]    := Chr(i+Ord('A'));
    Buffer[i+26] := Chr(i+Ord('a'));
  end;

  { Get the dimensions of the resulting string and return the average width }
  GetTextExtentPoint32(Canvas.Handle,Buffer,52,Size);
  Result := Size.cx div 52;
end;

function TMessageDialog.GetDialogUnitsY: Integer;
var
  i     : Integer;
  Buffer: array[0..51] of Char;
  Size  : TSize;
begin
  { Create an array of all small and capital letters }
  for i := 0 to 25 do begin
    Buffer[i]    := Chr(i+Ord('A'));
    Buffer[i+26] := Chr(i+Ord('a'));
  end;

  { Get the dimensions of the resulting string and return the height }
  GetTextExtentPoint32(Canvas.Handle,Buffer,52,Size);
  Result := Size.cy;
end;

{ ----------------------------------------------------------------------------
  Utilities
  ---------------------------------------------------------------------------- }
function Max(Value1, Value2: Integer): Integer;
begin
  { Return the greater of the two values }
  Result := Value1;
  if Value1 < Value2 then
    Result := Value2;
end;


{ MsgDlg function }
function MsgDlg(const Msg: String; AType: TMsgDlgType; AButtons: TMsgDlgButtons;
                HelpCtx: LongInt): Word;
const
  cHorzSpacing  = 8;
  cVertSpacing  = 8;
  cButtonWidth  = 50;
  cButtonHeight = 14;
var
  Dialog        : TMessageDialog;
  IconID        : PChar;
  TextRect      : TRect;
  Button        : TButton;
  Bevel         : TBevel;
  MessageWidth,
  MessageHeight,
  ButtonW,
  ButtonH,
  ButtonsWidth,
  IconW,
  BtnX,
  BtnY          : Integer;
  Btn,
  DefaultBtn,
  CancelBtn     : TMsgDlgBtn;
begin
  Dialog := TMessageDialog.CreateNew(Application);
  try
    with Dialog do begin
      BorderStyle := bsDialog;
      Canvas.Font := Font;
      Caption     := MsgDlgCaptions[AType];
      if Caption = '' then Caption := Application.Title;

      { Get the rectangle required to display the text }
      SetRect(TextRect, 0, 0, (Screen.Width div 8)*3, 0);
      DrawText(Canvas.Handle,PChar(Msg),-1,TextRect,DT_EXPANDTABS or DT_CALCRECT
               or DT_WORDBREAK);

      { Get the width of the message including the bevel }
      MessageWidth  := TextRect.Right+(2*cHorzSpacing);
      MessageHeight := TextRect.Bottom+(2*cVertSpacing);

      { Calc the width and height of a single button }
      ButtonW       := MulDiv(cButtonWidth,DialogUnitsX,4);
      ButtonH       := MulDiv(cButtonHeight,DialogUnitsY,8);

      { Calc the width of all buttons together }
      ButtonsWidth := 0;
      for Btn := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
        if Btn in AButtons then
          Inc(ButtonsWidth,ButtonW+4);
      Dec(ButtonsWidth,4);

      { Show the icon aligned to the top and the left }
      IconID := MsgDlgIcons[AType];
      IconW := cHorzSpacing;
      if IconID <> nil then begin
        with TImage.Create(Dialog) do begin
          Name := 'Image';
          Parent := Dialog;
          Picture.Icon.Handle := LoadIcon(0, IconID);
          SetBounds(cHorzSpacing,cVertSpacing,32,32);
        end;
        Inc(IconW,32+cVertSpacing);

        if (MessageHeight < 32) then MessageHeight := 32;
      end;

      { Insert the bevel next to the icon }
      Bevel := TBevel.Create(Dialog);
      with Bevel do begin
        Name := 'Frame';
        Parent := Dialog;
        Shape := bsFrame;
        SetBounds(IconW,cVertSpacing,2*cHorzSpacing+TextRect.Right,
                  MessageHeight);
        if ButtonsWidth > MessageWidth then
          Width := ButtonsWidth;
      end;

      { Insert the text into the bevel }
      with TLabel.Create(Dialog) do begin
        Name := 'Text';
        Parent := Dialog;
        WordWrap := True;
        AutoSize := False;
        Caption := Msg;
        SetBounds(IconW+cHorzSpacing,2*cVertSpacing,TextRect.Right,TextRect.Bottom);
      end;

      { Find the default button }
      if mbOk in AButtons then
        DefaultBtn := mbOk
      else if mbYes in AButtons then
        DefaultBtn := mbYes
      else
        DefaultBtn := mbRetry;

      { Find the cancel button }
      if mbCancel in AButtons then
        CancelBtn := mbCancel
      else if mbNo in AButtons then
        CancelBtn := mbNo
      else
        CancelBtn := mbOk;

      { Insert the buttons }
      BtnX := Bevel.Left+Bevel.Width-ButtonW;
      BtnY := Bevel.Top+Bevel.Height+cVertSpacing;
      for Btn := High(TMsgDlgBtn) downto Low(TMsgDlgBtn) do begin
        if Btn in AButtons then begin
          Button := TButton.Create(Dialog);
          with Button do begin
            Name := MsgDlgButtonNames[Btn];
            Caption := MsgDlgButtonCaptions[Btn];
            ModalResult := MsgDlgResults[Btn];
            Parent := Dialog;
            SetBounds(BtnX,BtnY,ButtonW,ButtonH);
            if Btn = DefaultBtn then begin
              Default := True;
              ActiveControl := Button;
            end;
            if Btn = CancelBtn then
              Cancel := True;
            if Btn = mbHelp then
              OnClick := HelpButtonClick;
          end;
          Dec(BtnX,ButtonW+4);
        end;
      end;

      { Set the correct size of the window and center it }
      ClientWidth := cHorzSpacing+IconW+Max(MessageWidth,ButtonsWidth);
      ClientHeight := cVertSpacing+MessageHeight+(2*cVertSpacing)+ButtonH;
      Left  := (Screen.Width-Width) div 2;
      Top   := (Screen.Height-Height) div 2;
    end;

    Application.ProcessMessages;
    MessageBeep(MsgDlgSounds[AType]);
    Result := Dialog.ShowModal;
  finally
    Dialog.Free;
  end;
end;

end.
